/*
 * clSemiLinkedList.c
 *
 *  Created on: 9 apr. 2015
 *      Author: Henk Mulder
 */
#include <CL/opencl.h>
#include <stdio.h>
#include <stdlib.h>

#include "clSemiLinkedList.h"

cl_mem _clSemiLinkedList_create_32(cl_context ctx, cl_command_queue cq) {
cl_mem gL;
cl_int error;

	gL = clCreateBuffer(ctx, CL_MEM_READ_WRITE, sizeof(clSemiLinkedList_32), NULL, &error);
	if(error!=CL_SUCCESS){
		printf("clSemiLinkedList: Could not allocate list on device.\n");
		return (cl_mem) 0;
	}
	clFinish(cq);
	return gL;
}

cl_mem _clSemiLinkedList_create_64(cl_context ctx, cl_command_queue cq) {
cl_mem gL;
cl_int error;

	gL = clCreateBuffer(ctx, CL_MEM_READ_WRITE, sizeof(clSemiLinkedList_64), NULL, &error);
	if(error!=CL_SUCCESS){
		printf("clSemiLinkedList: Could not allocate list on device.\n");
		return (cl_mem) 0;
	}
	clFinish(cq);
	return gL;
}

/**
 * Create a memory buffer for an entrypoint of an (initially empty) linked list for the given OpenCL environment.
 */
cl_mem clSemiLinkedList_create(cl_device_id dev, cl_context ctx,
		cl_command_queue cq, cl_program prg) {
	cl_int error;
	cl_uint bits;
	cl_kernel kernel;
	size_t threads = 1;
	cl_mem sll;

	error = clGetDeviceInfo(dev, CL_DEVICE_ADDRESS_BITS, sizeof(cl_uint), &bits,
			NULL);
	if (error) {
		printf("CLSemiLinkedList: could not discover device address space\n");
		return (cl_mem) 0;
	}

	 if(bits == 32) {
	 sll = _clSemiLinkedList_create_32(ctx, cq);
	 } else {
	 sll = _clSemiLinkedList_create_64(ctx, cq);
	 }
	/* Initialise kernel */
	kernel = clCreateKernel(prg, "clSemiLinkedList_init", &error);
	if(error != CL_SUCCESS) {
		printf("clSemiLinkedList: Could not create SemiLinkedList init kernel: %i\n", error);
		return (cl_mem) 0;
	}

	clSetKernelArg(kernel, 0, sizeof(cl_mem), &sll);

	error = clEnqueueNDRangeKernel(cq, kernel, 1, NULL, &threads, NULL, 0, NULL, NULL);
	if (error != CL_SUCCESS) {
		printf("clSemiLinkedList: Could not execute init kernel: %i\n", error);
		return (cl_mem) 0;
	}
	error = clFinish(cq);
	if(error != CL_SUCCESS) {
		printf("clSemiLinkedList: failed to initialise: %i\n", error);
		return (cl_mem) 0;
	}
	clReleaseKernel(kernel);

	return sll;
}


